#include <Ice/Ice.h>
#include <Ice/Service.h>

#include "srv.h"
#include "Plugin.hh"

#include <stdlib.h>
#include <dlfcn.h>
using std::string;

//--------------------------------------------------------------------
class SrvI : virtual public M::Srv {
public:

  SrvI();
  virtual ~SrvI();

  // from M::Srv

  virtual void f(const Ice::Current &) {}

private: // data

  Plugin * p_;
};


//----------------------------------------------------------------------
SrvI::SrvI()
  : p_(0)
{
  // Load plug-in

  string lib_path = "./libplugin.so";
  string fun_name = "plugin_new";

  void * lib_handler = dlopen(lib_path.c_str(), RTLD_NOW);

  typedef Plugin * (*p_new_func)();

  p_new_func func = (p_new_func) dlsym(lib_handler, fun_name.c_str());
  p_ = (*func)();
}

//----------------------------------------------------------------------
SrvI::~SrvI()
{
  delete p_;
}


//--------------------------------------------------------------------
class MyService : virtual public Ice::Service
{
public:

  // from Ice::Service
  virtual bool start(int argc, char * argv[]);
  virtual bool stop();

};

//--------------------------------------------------------------------
bool MyService::start(int argc, char * argv[])
{
  Ice::CommunicatorPtr ic = communicator();

  // Create adapter.

  string ice_addr = "default -p 13000";

  Ice::ObjectAdapterPtr adapter
    = ic->createObjectAdapterWithEndpoints("MyAdapter", ice_addr);

  // Create the Servant.

  Ice::ObjectPtr object = new SrvI;
  adapter->add(object, Ice::stringToIdentity("My"));

  // Activate the Servant.

  Ice::Trace out(ic->getLogger(), "SRV");
  out << "Server started. PID " << getpid();
  out.flush();

  adapter->activate();

  return true;
}

//--------------------------------------------------------------------
bool MyService::stop()
{
  Ice::Trace out(communicator()->getLogger(), "SRV");
  out << "Server stopped.";
  out.flush();
  return true;
}

//--------------------------------------------------------------------
int main(int argc, char * argv[])
{
  MyService svc;

  // Run the server.

  return svc.main(argc, argv);
}
